@@ -140,6 +140,11 @@ impl<T> NonEmpty<T> {
         self.tail.len() + 1
     }
 
+    /// Get the capacity of the list.
+    pub fn capacity(&self) -> usize {
+        self.tail.capacity() + 1
+    }
+
     /// Get the last element. Never fails.
     pub fn last(&self) -> &T {
         match self.tail.last() {
@@ -739,6 +744,46 @@ impl<T> IntoIterator for NonEmpty<T> {
     }
 }
 
+impl<'a, T> IntoIterator for &'a NonEmpty<T> {
+    type Item = &'a T;
+    type IntoIter = iter::Chain<iter::Once<&'a T>, std::slice::Iter<'a, T>>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        iter::once(&self.head).chain(self.tail.iter())
+    }
+}
+
+impl<T> std::ops::Index<usize> for NonEmpty<T> {
+    type Output = T;
+
+    /// ```
+    /// use nonempty::NonEmpty;
+    ///
+    /// let non_empty = NonEmpty::from((1, vec![2, 3, 4, 5]));
+    ///
+    /// assert_eq!(non_empty[0], 1);
+    /// assert_eq!(non_empty[1], 2);
+    /// assert_eq!(non_empty[3], 4);
+    /// ```
+    fn index(&self, index: usize) -> &T {
+        if index > 0 {
+            &self.tail[index - 1]
+        } else {
+            &self.head
+        }
+    }
+}
+
+impl<T> std::ops::IndexMut<usize> for NonEmpty<T> {
+    fn index_mut(&mut self, index: usize) -> &mut T {
+        if index > 0 {
+            &mut self.tail[index - 1]
+        } else {
+            &mut self.head
+        }
+    }
+}
+
 #[cfg(feature = "serialize")]
 pub mod serialize {
     use std::{convert::TryFrom, fmt};
@@ -791,6 +836,15 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_iter_syntax() {
+        let nonempty = NonEmpty::from((0, vec![1, 2, 3]));
+        for n in &nonempty {
+            assert_eq!(*n, *n); // Prove that we're dealing with references.
+        }
+        for _ in nonempty {}
+    }
+
     #[test]
     fn test_mutate_head() {
         let mut non_empty = NonEmpty::new(42);
